{ $Id: MYSQL.PP,v 1.1.1.1 2003/04/05 21:50:23 ele Exp $ }

unit mysql;

{$IFDEF FPC}
  {$mode objfpc}
{$ENDIF}

{$undef use_mysql_321} { if undefined, use mysql 3.23 interface }

{
  Import unit for the mysql header files.

  Translated form the original mysql.h by Michael Van Canneyt
  (michael@tfdec1.fys.kuleuven.ac.be)

  updated to mysql version 3.23 header files by Bernhard Steffen
  (bernhard.steffen@gmx.net)
  }
{$r+,i+,o+}

interface

uses mysql_com, mysql_version
       {$IFDEF Unix}
         ,Dl2
         ,Sysutils
       {$ENDIF}

       {$IFDEF Win32}
         ,Windows
       {$ENDIF};

{$ifdef win32}
Const mysqllib = 'libmysql';
{$else}
Const mysqllib = '/usr/lib/libmysqlclient.so';
{$endif}

{$ifndef win32}
{$linklib c}
{$linklib m}
{$endif}

{ All is 4-byte aligned on my machine. }
{$packrecords 4}

type
   my_bool = byte;
   gptr   = pchar;
   Socket = Longint;
   PCardinal = ^Cardinal;

{$ifdef linux}
Var
  mysql_port : ^cardinal;
  mysql_unix_port : ppchar;
{$endif}

{
#define IS_PRI_KEY(n)   ((n) & PRI_KEY_FLAG)
#define IS_NOT_NULL(n)  ((n) & NOT_NULL_FLAG)
#define IS_BLOB(n)      ((n) & BLOB_FLAG)
#define IS_NUM(t)       ((t) <= FIELD_TYPE_INT24)
}
Type
st_mysql_field = record
  name : Pchar;                 { Name of column }
  table : pchar;                { Table of column if column was a field }
  def: pchar;                   { Default value (set by mysql_list_fields) }
  ftype : enum_field_types;     { Type of field. See mysql_com.h for types }
  length : cardinal;            { Width of column }
  max_length : cardinal;        { Max width of selected set }
  flags : cardinal;             { Div flags }
  decimals : cardinal;  { Number of decimals in field }
end;
TMYSQL_FIELD = st_mysql_field;
PMYSQL_FIELD = ^TMYSQL_FIELD;

TMYSQL_ROW = PPchar;             { return data as array of strings }
TMYSQL_FIELD_OFFSET = cardinal;  { offset to current field }

{$ifndef oldmysql}
  my_ulonglong=qword;
{$else}
  my_longlong=cardinal;
{$endif}

PST_MYSQL_Rows = ^st_mysql_rows;
st_mysql_rows = Record
  next : pst_mysql_rows;                { list of rows }
  Data : TMYSQL_ROW;
end;
TMYSQL_ROWS = st_mysql_rows;
PMYSQL_ROWS = ^TMYSQL_ROWS;


TMYSQL_ROW_OFFSET = PMYSQL_ROWS;        { offset to current row }

st_mysql_data  = record
  rows   : my_ulonglong;
  fields : cardinal;
  data : PMYSQL_ROWS;
  alloc : TMEM_ROOT;
end;

TMYSQL_DATA = st_mysql_data;
PMYSQL_DATA = ^TMYSQL_DATA;

st_mysql_options = record
  connect_timeout,client_flag : cardinal;
  compress,named_pipe : my_bool;
  port : cardinal;
  host,init_command,user,password,unix_socket,db : pchar;
  my_cnf_file,my_cnf_group : pchar;
{$ifndef use_mysql_321}
  charset_dir, charset_name : pchar;
  use_ssl : my_bool;
  ssl_key, ssl_cert, ssl_ca, ssl_capath : pchar;
{$endif}
end;

{$ifndef use_mysql_321}
mysql_option = (MYSQL_OPT_CONNECT_TIMEOUT, MYSQL_OPT_COMPRESS,
                    MYSQL_OPT_NAMED_PIPE, MYSQL_INIT_COMMAND,
                    MYSQL_READ_DEFAULT_FILE, MYSQL_READ_DEFAULT_GROUP,
                    MYSQL_SET_CHARSET_DIR, MYSQL_SET_CHARSET_NAME);
{$endif}

mysql_status = (MYSQL_STATUS_READY,
                MYSQL_STATUS_GET_RESULT,
                MYSQL_STATUS_USE_RESULT);

{$ifndef use_mysql_321}
(*
charset_info_st = Record
        number : cardinal;
        name : pchar;
        ctype : pointer {uchar*};
        to_lower : pointer {uchar*};
        to_upper : pointer {uchar*};
        sort_order : pointer {uchar*};
        strxfrm_multiply : cardinal;

        { einige nicht nher definierte Felder }
        a, strxfrm, strnncoll, strnxfrm, like_range : pointer;
        mbmaxlen : cardinal;
        ismbchar, ismbhead, mbcharlen : pointer;
end;
*)
{$endif}

st_mysql = Record
  NET : TNET;                   { Communication parameters }
{$ifndef use_mysql_321}
  connector_fd : gptr;
{$endif}
  host,user,passwd,unix_socket,server_version,host_info,
  info,db : pchar;
  port,client_flag,server_capabilities : cardinal;
  protocol_version : cardinal;
  field_count : cardinal;
{$ifndef use_mysql_321}
  server_status : cardinal;
{$endif}
  thread_id : cardinal;         { Id for connection in server }
  affected_rows : my_ulonglong;
  insert_id : my_ulonglong;             { id if insert on table with NEXTNR }
  extra_info : my_ulonglong;            { Used by mysqlshow }
  packet_length : cardinal;
  status : mysql_status;
  fields : PMYSQL_FIELD;
  field_alloc : TMEM_ROOT;
  free_me : my_bool;            { If free in mysql_close }
  reconnect : my_bool;          { set to 1 if automatic reconnect }
  options : st_mysql_options;
{$ifndef use_mysql_321}
  scramble_buf : array[0..8] of char;
  charset : pointer { struct charset_info_st};
  server_language : cardinal;
{$endif}
end;
TMYSQL = st_mysql;
PMYSQL = ^TMYSQL;


st_mysql_res = record
  row_count : my_ulonglong;
  field_count, current_field : cardinal;
  fields :         PMYSQL_FIELD;
  data :           PMYSQL_DATA;
  data_cursor :    PMYSQL_ROWS;
  field_alloc :    TMEM_ROOT;
  row :            TMYSQL_ROW;                  { If unbuffered read }
  current_row :    TMYSQL_ROW;          { buffer to current row }
  lengths :        pcardinal;           { column lengths of current row }
  handle :         PMYSQL;              { for unbuffered reads }
  eof :            my_bool;                     { Used my mysql_fetch_row }
end;
TMYSQL_RES  = st_mysql_res;
PMYSQL_RES  = ^TMYSQL_RES;


procedure InitializeMySQL;

{ Translated Macros }

Function mysql_num_rows (res : PMYSQL_RES) : my_ulonglong;
Function mysql_num_fields(res : PMYSQL_RES) : Cardinal;
Function mysql_eof(res : PMYSQL_RES) : my_bool;
Function mysql_fetch_field_direct(res : PMYSQL_RES; fieldnr : Cardinal) : TMYSQL_FIELD;
Function mysql_fetch_fields(res : PMYSQL_RES) : PMYSQL_FIELD;
Function mysql_row_tell(res : PMYSQL_RES) : PMYSQL_ROWS;
Function mysql_field_tell(res : PMYSQL_RES) : Cardinal;
Function mysql_affected_rows(mysql : PMYSQL): my_ulonglong;
Function mysql_insert_id(mysql : PMYSQL): my_ulonglong;
Function mysql_errno(mysql : PMYSQL) : Cardinal;
Function mysql_info(mysql : PMYSQL): Pchar;
Function mysql_reload(mysql : PMYSQL) : Longint;
Function mysql_thread_id(mysql : PMYSQL) : Cardinal;
Function mysql_error(mysql : PMYSQL) : pchar;

{ Original functions }

type
  mysql_connect_func = function(mysql : PMYSQL; host,user,passwd: pchar) : PMYSQL; {$ifdef win32}stdcall{$else}cdecl{$endif};
(*
  mysql_get_client_info_func = function : pchar; {$ifdef win32}stdcall{$else}cdecl{$endif};
*)

mysql_get_client_info_func = function: PChar; stdcall;


  mysql_get_server_info_func = function(mysql : PMYSQL) : pchar; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_real_connect_func = function(mysql : PMYSQL; const host,user,passwd : pchar;
                                   port : cardinal;
                                   unix_socket : pchar;
                                   clientflag : cardinal) : PMYSQL;{$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_close_func = function(sock : PMYSQL) : longint; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_select_db_func = function(MYSQL : PMYSQL; db : Pchar) : longint; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_query_func = function(mysql : PMYSQL; q : pchar) : longint; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_real_query_func = function(mysql : PMYSQL; q : Pchar; length : longint) : longint; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_create_db_func = function(mysql : PMYSQL; db : pchar) : longint; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_drop_db_func = function(mysql : PMYSQL; DB : Pchar) : longint; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_shutdown_func = function(mysql : PMYSQL) : longint; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_dump_debug_info_func = function(mysql : PMYSQL) : longint; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_refresh_func = function(mysql : PMYSQL; refresh_options : cardinal) : longint; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_kill_func = function(mysql : PMYSQL; pid : Cardinal) : longint; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_stat_func = function(mysql : PMYSQL) : Pchar; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_get_host_info_func = function(mysql : PMYSQL) : pchar; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_get_proto_info_func = function(mysql : PMYSQL) : Cardinal; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_list_dbs_func = function(mysql : PMYSQL;wild : Pchar) : PMYSQL_RES; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_list_tables_func = function(mysql : PMYSQL;Wild : Pchar) : PMYSQL_RES; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_list_fields_func = function(mysql : PMYSQL; table,wild : pchar) : PMYSQL_RES; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_list_processes_func = function(mysql : PMYSQL) : PMYSQL_RES; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_store_result_func = function(mysql : PMYSQL) : PMYSQL_RES; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_use_result_func = function(mysql : PMYSQL) : PMYSQL_RES; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_row_seek_func = function(mysql : PMYSQL_RES; Offs: TMYSQL_ROW_OFFSET): TMYSQL_ROW_OFFSET; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_field_seek_func = function(musql : PMYSQL_RES;offs : TMYSQL_FIELD_OFFSET): TMYSQL_FIELD_OFFSET; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_fetch_row_func = function(mysql : PMYSQL_RES) : TMYSQL_ROW; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_fetch_lengths_func = function(mysql : PMYSQL_RES) : PCardinal; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_fetch_field_func = function(handle : PMYSQL_RES) : PMYSQL_FIELD; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_escape_string_func = function(escto,escfrom : pchar; length : Cardinal) : cardinal; {$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_debug_proc = procedure(debug : pchar);{$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_free_result_proc = procedure(res : PMYSQL_RES);{$ifdef win32}stdcall{$else}cdecl{$endif};
  mysql_data_seek_proc = procedure(mysql : PMYSQL_RES; offs : cardinal);{$ifdef win32}stdcall{$else}cdecl{$endif};


var mysql_connect: mysql_connect_func;
    mysql_get_client_info: mysql_get_client_info_func;
    mysql_get_server_info: mysql_get_server_info_func;
    mysql_real_connect: mysql_real_connect_func;
    mysql_close: mysql_close_func;
    mysql_select_db: mysql_select_db_func;
    mysql_query: mysql_query_func;
    mysql_real_query: mysql_real_query_func;
    mysql_create_db: mysql_create_db_func;
    mysql_drop_db: mysql_drop_db_func;
    mysql_shutdown: mysql_shutdown_func;
    mysql_dump_debug_info: mysql_dump_debug_info_func;
    mysql_refresh: mysql_refresh_func;
    mysql_kill: mysql_kill_func;
    mysql_stat: mysql_stat_func;
    mysql_get_host_info: mysql_get_host_info_func;
    mysql_get_proto_info: mysql_get_proto_info_func;
    mysql_list_dbs: mysql_list_dbs_func;
    mysql_list_tables: mysql_list_tables_func;
    mysql_list_fields: mysql_list_fields_func;
    mysql_list_processes: mysql_list_processes_func;
    mysql_store_result: mysql_store_result_func;
    mysql_use_result: mysql_use_result_func;
    mysql_row_seek: mysql_row_seek_func;
    mysql_field_seek: mysql_field_seek_func;
    mysql_fetch_row: mysql_fetch_row_func;
    mysql_fetch_lengths: mysql_fetch_lengths_func;
    mysql_fetch_field: mysql_fetch_field_func;
    mysql_escape_string: mysql_escape_string_func;
    mysql_debug: mysql_debug_proc;
    mysql_free_result: mysql_free_result_proc;
    mysql_data_seek: mysql_data_seek_proc;

implementation

{$IFDEF Unix}
var
  DlLibPtr: Pointer;
{$ENDIF}

Function  mysql_error(mysql : PMYSQL) : pchar;

begin
 mysql_error:=mysql^.net.last_error
end;

Function mysql_num_rows (res : PMYSQL_RES) : my_ulonglong;

begin
  mysql_num_rows:=res^.row_count
end;

Function mysql_num_fields(res : PMYSQL_RES) : Cardinal;

begin
  mysql_num_fields:=res^.field_count
end;

Function mysql_eof(res : PMYSQL_RES) : my_bool;

begin
  mysql_eof:=res^.eof
end;

Function mysql_fetch_field_direct(res : PMYSQL_RES; fieldnr : Cardinal) : TMYSQL_FIELD;

begin
  mysql_fetch_field_direct:=res^.fields[fieldnr];
end;

Function mysql_fetch_fields(res : PMYSQL_RES) : PMYSQL_FIELD;

begin
 mysql_fetch_fields:=res^.fields
end;

Function mysql_row_tell(res : PMYSQL_RES) : PMYSQL_ROWS;

begin
  mysql_row_tell:=res^.data_cursor
end;

Function mysql_field_tell(res : PMYSQL_RES) : Cardinal;

begin
  mysql_field_tell:=res^.current_field
end;

Function mysql_affected_rows(mysql : PMYSQL): my_ulonglong;

begin
  mysql_affected_rows:=mysql^.affected_rows
end;

Function mysql_insert_id(mysql : PMYSQL): my_ulonglong;

begin
  mysql_insert_id:=mysql^.insert_id
end;

Function mysql_errno(mysql : PMYSQL) : Cardinal;

begin
  mysql_errno:=mysql^.net.last_errno
end;

Function mysql_info(mysql : PMYSQL): Pchar;

begin
  mysql_info:=mysql^.info
end;

Function mysql_reload(mysql : PMYSQL) : Longint;

begin
  mysql_reload:=mysql_refresh(mysql,REFRESH_GRANT)
end;

Function mysql_thread_id(mysql : PMysql) : Cardinal;

begin
  mysql_thread_id:=mysql^.thread_id
end;


{$IFDEF Win32}
function DoLoadProc(DllName, ProcName: AnsiString): Pointer;
// function modelled after library loading of the JEDI units
var ModuleHandle: HMODULE;
begin
  {-- Initialize some variables ---------------------------------------------}
  Result := nil;

  {-- try to load the dll ---------------------------------------------------}
  ModuleHandle := GetModuleHandle(PChar(DllName));
  if (ModuleHandle = INVALID_HANDLE_VALUE) OR (GetLastError <> 0) then
    begin
      ModuleHandle := LoadLibrary(PChar(DllName));
    end; { if }

  {-- try to get the address ------------------------------------------------}
  if ModuleHandle <> INVALID_HANDLE_VALUE then
    begin
      Result := GetProcAddress(ModuleHandle, PChar(ProcName));
    end; { if }
end; { func. DoLoadProc }
{$ENDIF}

{$IFDEF UNIX}
function DoLoadProc(DllName, ProcName: AnsiString): Pointer;
// function modelled after library loading of the JEDI units
begin
  {-- Initialize some variables ---------------------------------------------}
  Result := nil;

  {-- try to load the dll ---------------------------------------------------}
  if DlLibPtr <> nil then
    begin
      Result := dlSym(DlLibPtr, PChar(ProcName));

writeln('[', procname, '] --> ', dlerror, ' -> ', Longint(Result));
    end; { if }
end; { func. DoLoadProc }
{$ENDIF}

procedure InitializeMySQL;
begin
  {$IFDEF Unix}
    DlLibPtr := DlOpen(mysqllib, RTLD_LAZY);
  {$ENDIF}

  mysql_connect := mysql_connect_func(DoLoadProc(mysqllib, 'mysql_connect'));
  mysql_get_client_info := mysql_get_client_info_func(DoLoadProc(mysqllib, 'mysql_get_client_info'));
  mysql_real_connect := mysql_real_connect_func(DoLoadProc(mysqllib, 'mysql_real_connect'));
  mysql_close := mysql_close_func(DoLoadProc(mysqllib, 'mysql_close'));
  mysql_select_db := mysql_select_db_func(DoLoadProc(mysqllib, 'mysql_select_db'));
  mysql_query := mysql_query_func(DoLoadProc(mysqllib, 'mysql_query'));
  mysql_real_query := mysql_real_query_func(DoLoadProc(mysqllib, 'mysql_real_query'));
  mysql_create_db := mysql_create_db_func(DoLoadProc(mysqllib, 'mysql_create_db'));
  mysql_drop_db := mysql_drop_db_func(DoLoadProc(mysqllib, 'mysql_drop_db'));
  mysql_shutdown := mysql_shutdown_func(DoLoadProc(mysqllib, 'mysql_shutdown'));
  mysql_dump_debug_info := mysql_dump_debug_info_func(DoLoadProc(mysqllib, 'mysql_dump_debug_info'));
  mysql_refresh := mysql_refresh_func(DoLoadProc(mysqllib, 'mysql_refresh'));
  mysql_kill := mysql_kill_func(DoLoadProc(mysqllib, 'mysql_kill'));
  mysql_stat := mysql_stat_func(DoLoadProc(mysqllib, 'mysql_stat'));
  mysql_get_server_info := mysql_get_server_info_func(DoLoadProc(mysqllib, 'mysql_get_server_info'));
  mysql_get_host_info := mysql_get_host_info_func(DoLoadProc(mysqllib, 'mysql_get_host_info'));
  mysql_get_proto_info := mysql_get_proto_info_func(DoLoadProc(mysqllib, 'mysql_get_proto_info'));
  mysql_list_dbs := mysql_list_dbs_func(DoLoadProc(mysqllib, 'mysql_list_dbs'));
  mysql_list_tables := mysql_list_tables_func(DoLoadProc(mysqllib, 'mysql_list_tables'));
  mysql_list_fields := mysql_list_fields_func(DoLoadProc(mysqllib, 'mysql_list_fields'));
  mysql_list_processes := mysql_list_processes_func(DoLoadProc(mysqllib, 'mysql_list_processes'));
  mysql_store_result := mysql_store_result_func(DoLoadProc(mysqllib, 'mysql_store_result'));
  mysql_use_result := mysql_use_result_func(DoLoadProc(mysqllib, 'mysql_use_result'));
  mysql_row_seek := mysql_row_seek_func(DoLoadproc(mysqllib, 'mysql_row_seek'));
  mysql_field_seek := mysql_field_seek_func(DoLoadProc(mysqllib, 'mysql_field_seek'));
  mysql_fetch_row := mysql_fetch_row_func(DoLoadProc(mysqllib, 'mysql_fetch_row'));
  mysql_fetch_lengths := mysql_fetch_lengths_func(DoLoadProc(mysqllib, 'mysql_fetch_lengths'));
  mysql_fetch_field := mysql_fetch_field_func(DoLoadProc(mysqllib, 'mysql_fetch_field'));
  mysql_escape_string := mysql_escape_string_func(DoLoadProc(mysqllib, 'mysql_escape_string'));
  mysql_debug := mysql_debug_proc(DoLoadProc(mysqllib, 'mysql_debug'));
  mysql_free_result := mysql_free_result_proc(DoLoadProc(mysqllib, 'mysql_free_result'));
  mysql_data_seek := mysql_data_seek_proc(DoLoadProc(mysqllib, 'mysql_data_seek'));

  {$IFDEF LINUX}
    mysql_port := DoLoadProc(mysqllib, 'mysql_port');
    mysql_unix_port := DoLoadProc(mysqllib, 'mysql_unix_port');
  {$ENDIF}

  {$IFDEF Unix}
    DlClose(DlLibPtr);
  {$ENDIF}
end; { proc. InitializeMySQL }


begin
  mysql_connect:= nil;
  mysql_real_connect:= nil;
  mysql_close:= nil;
  mysql_select_db:= nil;
  mysql_query:= nil;
  mysql_real_query:= nil;
  mysql_create_db:= nil;
  mysql_drop_db:= nil;
  mysql_shutdown:= nil;
  mysql_dump_debug_info:= nil;
  mysql_refresh:= nil;
  mysql_kill:= nil;
  mysql_stat:= nil;
  mysql_get_server_info:= nil;
  mysql_get_client_info:= nil;
  mysql_get_host_info:= nil;
  mysql_get_proto_info:= nil;
  mysql_list_dbs:= nil;
  mysql_list_tables:= nil;
  mysql_list_fields:= nil;
  mysql_list_processes:= nil;
  mysql_store_result:= nil;
  mysql_use_result:= nil;
  mysql_free_result:= nil;
  mysql_data_seek:= nil;
  mysql_row_seek:= nil;
  mysql_field_seek:= nil;
  mysql_fetch_row:= nil;
  mysql_fetch_lengths:= nil;
  mysql_fetch_field:= nil;
  mysql_escape_string:= nil;
  mysql_debug:= nil;
end.


{
  $Log: MYSQL.PP,v $
  Revision 1.1.1.1  2003/04/05 21:50:23  ele
  Initial import

  Revision 1.4  2002/09/10 19:44:28  michael
  + Changed stdcall to appropriate OS calling declaration

  Revision 1.3  2002/09/07 15:42:52  peter
    * old logs removed and tabs fixed

  Revision 1.2  2002/08/26 17:52:31  michael
  + Upgraded to 3.23

}
